from zs_utils.api.amazon.base_api import AmazonAPI


class GetOrderListAPI(AmazonAPI):
    """
    Docs:
    https://developer-docs.amazon.com/sp-api/docs/orders-api-v0-reference#get-ordersv0orders
    """

    http_method = "GET"
    resource_method = "orders/v0/orders"
    required_params = ["MarketplaceIds"]
    allowed_params = [
        "CreatedAfter",
        "CreatedBefore",
        "LastUpdatedAfter",
        "LastUpdatedBefore",
        "OrderStatuses",
        "FulfillmentChannels",
        "PaymentMethods",
        "BuyerEmail",
        "SellerOrderId",
        "MaxResultsPerPage",
        "EasyShipShipmentStatuses",
        "ElectronicInvoiceStatuses",
        "NextToken",
        "AmazonOrderIds",
        "ActualFulfillmentSupplySourceId",
        "IsISPU",
        "StoreChainStoreId",
        "EarliestDeliveryDateBefore",
        "EarliestDeliveryDateAfter",
        "LatestDeliveryDateBefore",
        "LatestDeliveryDateAfter",
    ]


class GetOrderAPI(AmazonAPI):
    """
    Docs:
    https://developer-docs.amazon.com/sp-api/docs/orders-api-v0-reference#get-ordersv0ordersorderid
    """

    http_method = "GET"
    resource_method = "orders/v0/orders/{orderId}"
    required_params = ["orderId"]


class GetOrderItemsAPI(AmazonAPI):
    """
    Docs:
    https://developer-docs.amazon.com/sp-api/docs/orders-api-v0-reference#get-ordersv0ordersorderidorderitems
    """

    http_method = "GET"
    resource_method = "orders/v0/orders/{orderId}/orderItems"
    required_params = ["orderId"]


class GetOrderAddressAPI(AmazonAPI):
    """
    Docs:
    https://developer-docs.amazon.com/sp-api/docs/orders-api-v0-reference#get-ordersv0ordersorderidaddress
    """

    http_method = "GET"
    resource_method = "orders/v0/orders/{orderId}/address"
    required_params = ["orderId"]


class UpdateShipmentAPI(AmazonAPI):
    """
    Docs:
    https://developer-docs.amazon.com/sp-api/docs/orders-api-v0-reference#post-ordersv0ordersorderidshipment
    """

    http_method = "POST"
    resource_method = "orders/v0/orders/{orderId}/shipment"
    required_params = ["orderId", "payload"]
    # payload = {
    #     "marketplaceId": "",
    #     "shipmentStatus": "",  # ReadyForPickup, PickedUp, RefusedPickup
    #     "orderItems": [{
    #         "orderItemId": "",
    #         "quantity": 1,
    #     }],
    # }


class ConfirmShipmentAPI(AmazonAPI):
    """
    Docs:
    https://developer-docs.amazon.com/sp-api/docs/orders-api-v0-reference#post-ordersv0ordersorderidshipmentconfirmation
    """

    http_method = "POST"
    resource_method = "orders/v0/orders/{orderId}/shipmentConfirmation"
    required_params = ["orderId", "payload"]
    # payload = {
    #     "marketplaceId": "",
    #     "packageDetail": {
    #         "packageReferenceId": None,  # r
    #         "carrierCode": None,  # r
    #         "carrierName": None,
    #         "shippingMethod": None,
    #         "trackingNumber": None,  # r
    #         "shipDate": None,  # r
    #         "shipFromSupplySourceId": None,
    #         "orderItems": [{}],  # r (see above)
    #     },
    #     "codCollectionMethod": "DirectPayment",  # JP only
    # }



logo = "logo-1.png"

base_template = """
    <!DOCTYPE html>
    <html lang="ru">
    <head>
        <meta charset="UTF-8">
        <style type="text/css">
            @font-face {{{{
                font-family: Trebuchet;
                font-style: normal;
                font-weight: 400;
                src: url('fonts/TrebuchetMS.woff') format('woff');
            }}}}
            .button {{{{
                text-decoration: none;
                background-color: #43D6C1;
                font-size: 18px;
                font-weight: bold;
                color: #ffffff;
                padding: 8px 30px;
                border-radius: 50px;
            }}}}
        </style>
    </head>
    <body>
        <table border="0" cellpadding="0" cellspacing="0" width="100%" style="border-collapse:collapse;word-break:normal;word-wrap:normal;">
            <tbody>
                <tr>
                    <td align="center">
                        <table border="0" cellpadding="0" cellspacing="0" width="100%" style="border-collapse:collapse;word-break:normal;word-wrap:normal; font-family: 'Trebuchet', sans-serif;max-width:600px;width:100%">
                            <tbody>
                            <tr>
                                <td align="center" style="word-break:normal;word-wrap:normal">
                                    <table bgcolor="#ffffff" border="0" cellpadding="0" cellspacing="0" width="100%" style="box-sizing:border-box;max-width:420px;width:100%">
                                        <tbody>
                                        <tr>
                                            <td style="padding: 20px 0">
                                                <img src="{static_url}{logo}" alt="logo">
                                            </td>
                                        </tr>
                                        <tr>
                                            <td align="center" style="padding: 20px 0 0 0">
                                                <img src="{static_url}{email_icon}" alt="icon_email">
                                            </td>
                                        </tr>
                                        <tr>
                                            <td align="center" style="padding: 10px 0 30px 0">
                                                <h1 style="color: #37373C;font-family: 'Trebuchet', sans-serif;font-weight: bold;font-size: 24px;line-height: 140%; letter-spacing: 1.3px; margin-bottom: 0">
                                                    {title}
                                                </h1>
                                            </td>
                                        </tr>
                                        </tbody>
                                    </table>
                                </td>
                            </tr>
                            <tr>
                                <td style="font-size: 13px">
                                    {body}
                                </td>
                            </tr>
                            <tr>
                                <td>
                                    <hr color="#EAE7FF">
                                </td>
                            </tr>
                            <tr>
                                <td align="center" style="word-break:normal;word-wrap:normal">
                                    <table bgcolor="#ffffff" border="0" cellpadding="0" cellspacing="0" width="100%" style="box-sizing:border-box;width:100%; font-size: 13px; color: #888888; padding: 0 20px">
                                        <tbody>
                                        <tr>
                                            <td style="padding: 40px 0" align="left">
                                                <table>
                                                    <tbody>
                                                        {cheers}
                                                    </tbody>
                                                </table>
                                            </td>
                                            <td style="padding: 40px 0" align="right">
                                                <a href="https://www.facebook.com/thezonesmart" style="text-decoration: none; margin: 0 2px">
                                                    <img src="{static_url}sn_icons/facebook-icon.png" alt="facebook">
                                                </a>
                                                <a href="https://www.instagram.com/zonesmart.ru/" style="text-decoration: none; margin: 0 2px">
                                                    <img src="{static_url}sn_icons/instagram-icon.png" alt="instagram">
                                                </a>
                                                <a href="https://www.linkedin.com/company/zonesmart/" style="text-decoration: none; margin: 0 2px">
                                                    <img src="{static_url}sn_icons/linkedin-icon.png" alt="linkedin">
                                                </a>
                                                <a href="https://www.youtube.com/channel/UCYV_W03i-Ygoh_6-ENW-33Q" style="text-decoration: none; margin: 0 2px">
                                                    <img src="{static_url}sn_icons/youtube-icon.png" alt="youtube">
                                                </a>
                                            </td>
                                        </tr>
                                        <tr>
                                            <td colspan="2" style="padding: 20px 0 15px" align="left">
                                                <hr color="#EAE7FF">
                                            </td>
                                        </tr>
                                        <tr>
                                            <td colspan="2" align="center" style="padding: 20px 0 50px; font-size: 10px; line-height: 132%">
                                                <div style="max-width: 300px; width: 100%">
                                                    {footer}
                                                </div>
                                            </td>
                                        </tr>
                                        </tbody>
                                    </table>
                                </td>
                            </tr>
                            </tbody>
                        </table>
                    </td>
                </tr>
            </tbody>
        </table>
    </body>
    </html>
"""

cheers_team_email_ru = """
    <tr>
        <td>
            Спасибо за чтение,
        </td>
    </tr>
    <tr>
        <td style="color: #3BD0BC;">
            Команда Zonesmart
        </td>
    </tr>
"""



from zs_utils.email_notification.templates.repricer import base

icon = "payment_remind_icon.png"

title_ru = "Ошибка при продлении подписки"

body_ru = """
    <table border="0" cellpadding="0" cellspacing="0" width="100%" align="center" style="color: #515151; box-sizing:border-box;max-width: 420px;width:100%; font-size: 13px;line-height: 149%; padding: 0 20px">
        <tbody>
        <tr>
            <td style="padding: 0 0 40px">
                При продлении подписки произошла ошибка. В связи с этим синхронизация с маркетплейсами будет отключена.
            </td>
        </tr>
        <tr>
            <td align="center" style="padding: 0 0 40px">
                <a href="{dashboard_url}" class="button" style="color: #ffffff">
                    Перейти в Zonesmart
                </a>
            </td>
        </tr>
        </tbody>
    </table>
"""

cheers_ru = base.cheers_team_email_ru



from zs_utils.api.amazon.base_api import AmazonAPI


class GetCategoryTreeAPI(AmazonAPI):
    """
    Docs:
    https://developer-docs.amazon.com/sp-api/docs/catalog-items-api-v0-reference#listcatalogcategories
    """

    http_method = "GET"
    resource_method = "catalog/v0/categories"
    required_params = ["MarketplaceId"]
    allowed_params = [
        "ASIN",
        "SellerSKU",
    ]


class GetProductTypeDefinitionAPI(AmazonAPI):
    """
    Docs:
    https://developer-docs.amazon.com/sp-api/docs/product-type-definitions-api-v2020-09-01-reference#get-definitions2020-09-01producttypesproducttype
    """

    http_method = "GET"
    resource_method = "definitions/2020-09-01/productTypes/{productType}"
    required_params = ["productType", "marketplaceIds"]
    allowed_params = [
        "sellerId",
        "productTypeVersion",
        "requirements",
        "requirementsEnforced",
        "locale",
    ]



from django.contrib.postgres.fields import ArrayField
from django.db import models
from model_utils.models import TimeStampedModel, UUIDModel

from zs_utils.data.enums import LANGUAGES
from zs_utils.file_utils import get_email_notification_file_upload_path
from zs_utils.json_utils import CustomJSONEncoder


class EmailNotification(UUIDModel, TimeStampedModel):
    """
    Модель email-уведомления (письма)
    """

    sender = models.CharField(max_length=64, verbose_name="Отправитель")
    receivers = ArrayField(models.CharField(max_length=64), verbose_name="Получатели")
    language = models.CharField(max_length=2, choices=LANGUAGES, verbose_name="Язык")
    template_name = models.CharField(max_length=64, verbose_name="Шаблон")
    template_params = models.JSONField(
        blank=True,
        default=dict,
        encoder=CustomJSONEncoder,
        verbose_name="Параметры шаблона",
    )

    is_urgent = models.BooleanField(default=False, verbose_name="Уведомление срочное")
    is_sent = models.BooleanField(default=False, verbose_name="Уведомление отправлено")

    class Meta:
        ordering = ["-created"]


class EmailNotificationFile(UUIDModel):
    """
    Вложения для email-уведомлений
    """

    email_notification = models.ForeignKey(
        EmailNotification,
        on_delete=models.CASCADE,
        related_name="files",
        related_query_name="file",
        verbose_name="Уведомление",
    )
    name = models.CharField(max_length=255, verbose_name="Имя файла", default="")
    file = models.FileField(upload_to=get_email_notification_file_upload_path, verbose_name="Файл")

    class Meta:
        ordering = ["name"]



from zs_utils.email_notification.templates.repricer import base

icon = "new_message_icon.png"

title_ru = "Новое сообщение от пользователя Zonesmart Repricer"

body_ru = """
    <table border="0" cellpadding="0" cellspacing="0" width="100%" align="center" style="color: #515151; box-sizing:border-box;max-width: 420px;width:100%; font-size: 13px;line-height: 149%; padding: 0 20px">
        <tbody>
        <tr>
            <td style="padding: 0 0 30px">
                Вам поступило новое сообщение от пользователя <span style="color: #3BD0BC; font-weight: bold">Zonesmart Repricer</span>.
            </td>
        </tr>
        <tr>
            <td style="padding: 0 0 20px">
                <hr color="#EAE7FF">
            </td>
        </tr>
        <tr>
            <td>
                <table border="0" cellpadding="0" cellspacing="0" width="100%" style="color: #515151; box-sizing:border-box;width:100%; font-size: 13px;line-height: 149%;">
                    <tbody>
                    <tr>
                        <td style="padding: 0 10px 20px">
                            <img src="{user_img_url}" alt="user_img_url" width="90px">
                        </td>
                        <td style="padding: 0 10px 20px">
                            <span style="font-weight: 700">{first_name}</span>
                            <p>{message_text}</p>
                        </td>
                    </tr>
                    </tbody>
                </table>
            </td>
        </tr>
        <tr>
            <td style="padding: 0 0 40px">
                <hr color="#EAE7FF">
            </td>
        </tr>
        <tr>
            <td align="center" style="padding: 0 0 40px">
                <a href="{ticket_url}" class="button" style="color: #ffffff">
                    Перейти в чат
                </a>
            </td>
        </tr>
        </tbody>
    </table>
"""

cheers_ru = base.cheers_team_email_ru



from zs_utils.email_notification.templates.landing import base

icon = "icon_email.png"

subject_ru = "Новая заявка zonesmart.ru"

title_ru = "Поступила новая заявка с {url}"

body_ru = """
    <table border="0" cellpadding="0" cellspacing="0" width="100%" align="center" style="color: #515151; box-sizing:border-box;max-width: 420px;width:100%; font-size: 14px;line-height: 149%; padding: 0 20px">
        <tbody>
        <tr>
            <td>
                Время: {created_at} UTC
            </td>
        </tr>
        <tr>
            <td>
                Имя: {name}
            </td>
        </tr>
        <tr>
            <td>
                Телефон: {phone}
            </td>
        </tr>
        <tr>
            <td>
                Статус: Передано в Kokoc CRM
            </td>
        </tr>
        </tbody>
    </table>
"""

cheers_ru = base.cheers_team_email_ru



from zs_utils.api.amazon.base_api import AmazonAPI


class GetFeedAPI(AmazonAPI):
    """
    Docs:
    https://developer-docs.amazon.com/sp-api/docs/feeds-api-v2021-06-30-reference#post-feeds2021-06-30feeds
    """

    http_method = "GET"
    resource_method = "feeds/2021-06-30/feeds/{feedId}"
    required_params = ["feedId"]


class CreateFeedDocumentAPI(AmazonAPI):
    """
    Docs:
    https://developer-docs.amazon.com/sp-api/docs/feeds-api-v2021-06-30-reference#post-feeds2021-06-30documents
    """

    http_method = "POST"
    resource_method = "feeds/2021-06-30/documents"
    required_params = [
        "payload"
    ]  # POST_ORDER_ACKNOWLEDGEMENT_DATA - cancel order, POST_INVENTORY_AVAILABILITY_DATA - update stocks


class UploadFeedDataAPI(AmazonAPI):
    """
    Docs:
    https://developer-docs.amazon.com/sp-api/docs/feeds-api-v2021-06-30-reference#createfeeddocumentresponse
    """

    http_method = "PUT"
    required_params = ["payload"]


class CreateFeedAPI(AmazonAPI):
    """
    Docs:
    https://developer-docs.amazon.com/sp-api/docs/feeds-api-v2021-06-30-reference#post-feeds2021-06-30feeds
    """

    http_method = "POST"
    resource_method = "feeds/2021-06-30/feeds"
    required_params = ["payload"]



from zs_utils.api.base_api import BaseAPI


class AmazonAPI(BaseAPI):
    production_api_url = "https://sellingpartnerapi-{region}.amazon.com/"
    region_to_marketplace_ids = {
        "na": [
            "A2EUQ1WTGCTBG2",
            "ATVPDKIKX0DER",
            "A1AM78C64UM0Y8",
            "A2Q3Y263D00KWC",
        ],
        "eu": [
            "A1RKKUPIHCS9HS",
            "A1F83G8C2ARO7P",
            "A13V1IB3VIYZZH",
            "AMEN7PMS3EDWL",
            "A1805IZSGTT6HS",
            "A1PA6795UKMFR9",
            "APJ6JRA9NG5V4",
            "A2NODRKZP88ZB9",
            "AE08WJ6YKNBMC",
            "A1C3SOZRARQ6R3",
            "ARBP9OOSHTCHU",
            "A33AVAJ2PDY3EV",
            "A17E79C6D8DWNP",
            "A2VIGQ35RCS4UG",
            "A21TJRUUN4KGV",
        ],
        "fe": [
            "A19VAU5U5O7RUS",
            "A39IBJ37TRP1C6",
            "A1VC38T7YXB528",
        ],
    }

    def __init__(self, access_token: str, marketplace_id: str, url: str = None, **kwargs):
        super().__init__(**kwargs)

        for region, marketplace_ids in self.region_to_marketplace_ids.items():
            if marketplace_id in marketplace_ids:
                self.region = region
                break
        else:
            all_marketplace_ids = [
                item for marketplace_ids in self.region_to_marketplace_ids.values() for item in marketplace_ids
            ]
            raise ValueError("Доступные значения для 'marketplace_id': " + ", ".join(all_marketplace_ids))

        self.production_api_url = url if url else self.production_api_url.format(region=region)
        self.access_token = access_token

    @property
    def headers(self) -> dict:
        headers = {
            "Content-Type": "application/json",
            "x-amz-access-token": self.access_token,
        }
        return headers

    def get_payload(self, params: dict) -> dict | list:
        return params.pop("payload", {})

    def get_request_params(self, **kwargs) -> dict:
        """
        Получение всех параметров, необходимых для запроса (url, headers, params, json, files)
        """

        request_params = {
            "url": self.build_url(kwargs),
            "headers": self.headers,
        }

        clean_params = self.get_clean_params(kwargs)

        files = clean_params.pop("files", None)
        if files:
            request_params["files"] = files

        if self.http_method in ["POST", "PUT", "PATCH"]:
            request_params["json"] = self.get_payload(params=clean_params)
        request_params["params"] = clean_params
        return request_params



from zs_utils.api.amazon.base_api import AmazonAPI


class GetListingBySkuAPI(AmazonAPI):
    """
    Docs:
    https://developer-docs.amazon.com/sp-api/docs/listings-items-api-v2021-08-01-reference#get-listings2021-08-01itemsselleridsku
    """

    http_method = "GET"
    resource_method = "listings/2021-08-01/items/{sellerId}/{sku}"
    required_params = ["marketplaceIds"]
    allowed_params = [
        "issueLocale",
        "includedData",
    ]


class GetListingListAPI(AmazonAPI):
    """
    Docs:
    https://developer-docs.amazon.com/sp-api/docs/catalog-items-api-v2022-04-01-reference#get-catalog2022-04-01items
    """

    http_method = "GET"
    resource_method = "/catalog/2022-04-01/items"
    required_params = [
        "marketplaceIds",
        "identifiersType",
        "identifiers",
    ]
    allowed_params = [
        "includedData",
        "locale",
        "keywords",
        "brandNames",
        "classificationIds",
        "pageSize",
        "pageToken",
        "keywordsLocale",
    ]


# allows you to update the attributes of a given SKU.
class PatchListingAPI(AmazonAPI):
    """
    Docs:
    https://developer-docs.amazon.com/sp-api/docs/listings-items-api-v2021-08-01-reference#patch-listings2021-08-01itemsselleridsku
    """

    http_method = "PATCH"
    resource_method = "listings/2021-08-01/items/{sellerId}/{sku}"
    required_params = [
        "marketplaceIds",
        "sku",
        "sellerId",
        "payload",
    ]
    allowed_params = [
        "includedData",
        "mode",
        "issueLocale",
    ]


# allows you to create a given SKU and update attributes like price, inventory, or both
class CreateOrUpdateListingAPI(AmazonAPI):
    """
    Docs:
    https://developer-docs.amazon.com/sp-api/docs/listings-items-api-v2021-08-01-reference#put-listings2021-08-01itemsselleridsku
    """

    http_method = "PUT"
    resource_method = "listings/2021-08-01/items/{sellerId}/{sku}"
    required_params = [
        "marketplaceIds",
        "sku",
        "sellerId",
        "payload",
    ]
    allowed_params = [
        "includedData",
        "mode",
        "issueLocale",
    ]


# allows you to delete a given SKU
class DeleteListingAPI(AmazonAPI):
    """
    Docs:
    https://developer-docs.amazon.com/sp-api/docs/listings-items-api-v2021-08-01-reference#delete-listings2021-08-01itemsselleridsku
    """

    http_method = "DELETE"
    resource_method = "listings/2021-08-01/items/{sellerId}/{sku}"
    required_params = [
        "marketplaceIds",
        "sku",
        "sellerId",
    ]
    allowed_params = [
        "issueLocale",
    ]


class GetInventoriesAPI(AmazonAPI):
    """
    Docs:
    https://developer-docs.amazon.com/sp-api/docs/fbainventory-api-v1-reference#get-fbainventoryv1summaries
    """

    http_method = "GET"
    resource_method = "fba/inventory/v1/summaries"
    required_params = [
        "marketplaceIds",
        "granularityType",
        "granularityId",
    ]
    allowed_params = [
        "details",
        "startDateTime",
        "sellerSkus",
        "sellerSku",
        "nextToken",
    ]


class GetListingPriceAPI(AmazonAPI):
    """
    Docs:
    https://developer-docs.amazon.com/sp-api/docs/product-pricing-api-v0-reference#get-productspricingv0price
    """

    http_method = "GET"
    resource_method = "products/pricing/v0/price"
    required_params = [
        "MarketplaceId",
        "ItemType",
    ]
    allowed_params = [
        "Asins",
        "ItemCondition",
        "OfferType",
    ]



from celery import current_app as app

from zs_utils.base_support import services


@app.task(queue="low")
def close_inactive_tickets_task(**kwargs):
    """
    Закрытие всех неактивных обращений в поддержку для всех пользователей
    """
    services.CommonSupportTicketService.close_inactive_tickets()



from zs_utils.email_notification.templates.repricer import base

icon = "icon_email.png"

subject_ru = "Изменение email"

title_ru = "Подтвердите изменение email адреса"

body_ru = """
    <table border="0" cellpadding="0" cellspacing="0" width="100%" align="center" style="color: #515151; box-sizing:border-box;max-width: 420px;width:100%; font-size: 13px;line-height: 149%; padding: 0 20px">
        <tbody>
        <tr>
            <td style="padding: 0 0 10px">
                Здравствуйте, <span style="color: #3BD0BC; font-weight: bold">{first_name}</span>!
            </td>
        </tr>
        <tr>
            <td style="padding: 0 0 40px">
                Для изменения адреса электронной почты вашего аккаунта <span style='color: #3BD0BC; font-weight: bold'>Zonesmart</span>, пожалуйста, введите данный код:
            </td>
        </tr>
        <tr>
            <td style="padding: 0 0 40px">
                <span style='color: #3BD0BC; font-weight: bold; font-size: 16px'>{code}</span>
            </td>
        </tr>
        <tr>
            <td style="padding: 0 0 40px">
                Если вы не просили нас изменить email адрес, то просто проигнорируйте\nэто письмо. Ваша учетная запись в безопасности.
            </td>
        </tr>
        </tbody>
    </table>
"""

cheers_ru = base.cheers_team_email_ru

footer_ru = """
    Это письмо было отправлено, потому что вы изменили email на <a href='https://ar.zonesmart.com' style='color: #3BD0BC; text-decoration: none'>Zonesmart</a>
"""



# Generated by Django 4.1.5 on 2024-10-07 17:58

import django.contrib.postgres.fields
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields
import uuid
import zs_utils.file_utils
import zs_utils.json_utils


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="EmailNotification",
            fields=[
                (
                    "created",
                    model_utils.fields.AutoCreatedField(
                        default=django.utils.timezone.now, editable=False, verbose_name="created"
                    ),
                ),
                (
                    "modified",
                    model_utils.fields.AutoLastModifiedField(
                        default=django.utils.timezone.now, editable=False, verbose_name="modified"
                    ),
                ),
                (
                    "id",
                    model_utils.fields.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False),
                ),
                ("sender", models.CharField(max_length=64, verbose_name="Отправитель")),
                (
                    "receivers",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.CharField(max_length=64), size=None, verbose_name="Получатели"
                    ),
                ),
                (
                    "language",
                    models.CharField(
                        choices=[
                            ("en", "Английский"),
                            ("ru", "Русский"),
                            ("de", "Немецкий"),
                            ("zh", "Китайский"),
                            ("pl", "Польский"),
                            ("fr", "Французский"),
                            ("it", "Итальянский"),
                            ("pt", "Португальский"),
                            ("es", "Испанский"),
                            ("ja", "Японский"),
                            ("th", "Тайский"),
                            ("nl", "Голландский"),
                            ("ab", "Абхазский"),
                            ("aa", "Афар"),
                            ("af", "Африкаанс"),
                            ("ak", "Акан"),
                            ("sq", "Албанский"),
                            ("am", "Амхарский"),
                            ("ar", "Арабский"),
                            ("an", "Арагонский"),
                            ("hy", "Армянский"),
                            ("as", "Ассамский"),
                            ("av", "Аварский"),
                            ("ae", "Авестийский"),
                            ("ay", "Аймара"),
                            ("az", "Азербайджанский"),
                            ("bm", "Бамбара"),
                            ("ba", "Башкирский"),
                            ("eu", "Баскский"),
                            ("be", "Белорусский"),
                            ("bn", "Бенгальский"),
                            ("bh", "Бихари"),
                            ("bi", "Бислама"),
                            ("bs", "Боснийский"),
                            ("br", "Бретонский"),
                            ("bg", "Болгарский"),
                            ("my", "Бирманский"),
                            ("ca", "Каталанский"),
                            ("ch", "Чаморро"),
                            ("ce", "Чеченский"),
                            ("ny", "Ньянджа"),
                            ("cv", "Чувашский"),
                            ("kw", "Корнский"),
                            ("co", "Корсиканский"),
                            ("cr", "Кри"),
                            ("hr", "Хорватский"),
                            ("cs", "Чешский"),
                            ("da", "Датский"),
                            ("dv", "Дивехи"),
                            ("dz", "Дзонг-Кэ"),
                            ("eo", "Эсперанто"),
                            ("et", "Эстонский"),
                            ("ee", "Эве"),
                            ("fo", "Фарерский"),
                            ("fj", "Фиджи"),
                            ("fi", "Финский"),
                            ("ff", "Фулах"),
                            ("gl", "Галисийский"),
                            ("ka", "Грузинский"),
                            ("el", "Новогреческий (1453-)"),
                            ("gn", "Гуарани"),
                            ("gu", "Гуджарати"),
                            ("ht", "Гаитянский"),
                            ("ha", "Хауса"),
                            ("he", "Иврит"),
                            ("hz", "Гереро"),
                            ("hi", "Хинди"),
                            ("ho", "Хири Моту"),
                            ("hu", "Венгерский"),
                            ("ia", "Интерлингва (Ассоциация Международного Вспомогательного Языка)"),
                            ("id", "Индонезийский"),
                            ("ie", "Интерлингве"),
                            ("ga", "Ирландский"),
                            ("ig", "Игбо"),
                            ("ik", "Инулиак"),
                            ("io", "Идо"),
                            ("is", "Исландский"),
                            ("iu", "Инуктитут"),
                            ("jv", "Яванский"),
                            ("kl", "Калааллисут"),
                            ("kn", "Каннада"),
                            ("kr", "Канури"),
                            ("ks", "Кашмири"),
                            ("kk", "Казахский"),
                            ("km", "Кхмерский Центральный"),
                            ("ki", "Кикуйю"),
                            ("rw", "Киньяруанда"),
                            ("ky", "Киргизский"),
                            ("kv", "Коми"),
                            ("kg", "Конго"),
                            ("ko", "Корейский"),
                            ("ku", "Курдский"),
                            ("kj", "Киньяма"),
                            ("la", "Латынь"),
                            ("lb", "Люксембургский"),
                            ("lg", "Ганда"),
                            ("li", "Лимбурган"),
                            ("ln", "Лингала"),
                            ("lo", "Лаосский"),
                            ("lt", "Литовский"),
                            ("lu", "Луба-Катанга"),
                            ("lv", "Латышский"),
                            ("gv", "Мэнкский"),
                            ("mk", "Македонский"),
                            ("mg", "Малагасийский"),
                            ("ms", "Малайский (Макроязык)"),
                            ("ml", "Малаялам"),
                            ("mt", "Мальтийский"),
                            ("mi", "Маори"),
                            ("mr", "Маратхи"),
                            ("mh", "Маршалльский"),
                            ("mn", "Монгольский"),
                            ("na", "Науру"),
                            ("nv", "Навахо"),
                            ("nb", "Норвежский Букмол"),
                            ("nd", "Северный Ндебеле"),
                            ("ne", "Непальский (макроязык)"),
                            ("ng", "Ндунга"),
                            ("nn", "Норвежский Нюнорск"),
                            ("no", "Норвежский"),
                            ("ii", "Сычуань Йи"),
                            ("nr", "Южный Ндебеле"),
                            ("oc", "Окситанский (пост 1500)"),
                            ("oj", "Оджибва"),
                            ("cu", "Церковнославянский"),
                            ("om", "Оромо"),
                            ("or", "Ория (макроязык)"),
                            ("os", "Осетинский"),
                            ("pa", "Панджаби"),
                            ("pi", "Пали"),
                            ("fa", "Персидский"),
                            ("ps", "Пушту"),
                            ("qu", "Кечуа"),
                            ("rm", "Ретороманский"),
                            ("rn", "Рунди"),
                            ("ro", "Румынский"),
                            ("sa", "Санскрит"),
                            ("sc", "Сардинский"),
                            ("sd", "Синдхи"),
                            ("se", "Северные Саамы"),
                            ("sm", "Самоанский"),
                            ("sg", "Санго"),
                            ("sr", "Сербский"),
                            ("gd", "Шотландский Гэльский"),
                            ("sn", "Шона"),
                            ("si", "Сингальский"),
                            ("sk", "Словацкий"),
                            ("sl", "Словенский"),
                            ("so", "Сомали"),
                            ("st", "Южный Сото"),
                            ("su", "Сунданский"),
                            ("sw", "Суахили (Макроязык)"),
                            ("ss", "Свати"),
                            ("sv", "Шведский"),
                            ("ta", "Тамильский"),
                            ("te", "Телугу"),
                            ("tg", "Таджикский"),
                            ("ti", "Тигринья"),
                            ("bo", "Тибетский"),
                            ("tk", "Туркменский"),
                            ("tl", "Тагальский"),
                            ("tn", "Тсвана"),
                            ("to", "Тонга (Острова Тонга)"),
                            ("tr", "Турецкий"),
                            ("ts", "Тсонга"),
                            ("tt", "Татарский"),
                            ("tw", "Тви"),
                            ("ty", "Таитянский"),
                            ("ug", "Уйгурский"),
                            ("uk", "Украинский"),
                            ("ur", "Урду"),
                            ("uz", "Узбекский"),
                            ("ve", "Венда"),
                            ("vi", "Вьетнамский"),
                            ("vo", "Волапюк"),
                            ("wa", "Валлун"),
                            ("cy", "Валлийский"),
                            ("wo", "Волоф"),
                            ("fy", "Западно-фризский"),
                            ("xh", "Коса"),
                            ("yi", "Идиш"),
                            ("yo", "Йоруба"),
                            ("za", "Чжуан"),
                            ("zu", "Зулусский"),
                        ],
                        max_length=2,
                        verbose_name="Язык",
                    ),
                ),
                ("template_name", models.CharField(max_length=64, verbose_name="Шаблон")),
                (
                    "template_params",
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=zs_utils.json_utils.CustomJSONEncoder,
                        verbose_name="Параметры шаблона",
                    ),
                ),
                ("is_urgent", models.BooleanField(default=False, verbose_name="Уведомление срочное")),
                ("is_sent", models.BooleanField(default=False, verbose_name="Уведомление отправлено")),
            ],
            options={
                "ordering": ["-created"],
            },
        ),
        migrations.CreateModel(
            name="EmailNotificationFile",
            fields=[
                (
                    "id",
                    model_utils.fields.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False),
                ),
                ("name", models.CharField(default="", max_length=255, verbose_name="Имя файла")),
                (
                    "file",
                    models.FileField(
                        upload_to=zs_utils.file_utils.get_email_notification_file_upload_path, verbose_name="Файл"
                    ),
                ),
                (
                    "email_notification",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="files",
                        related_query_name="file",
                        to="email_notification.emailnotification",
                        verbose_name="Уведомление",
                    ),
                ),
            ],
            options={
                "ordering": ["name"],
            },
        ),
    ]



from django.conf import settings
from django.contrib.auth import get_user_model
from rest_framework import serializers


__all__ = [
    "UserLightSerializer",
]

User = get_user_model()


class UserLightSerializer(serializers.ModelSerializer):
    class Meta:
        model = User
        fields = getattr(
            settings,
            "USER_LIGHT_SERIALIZER_FIELDS",
            [
                "id",
                "email",
                "is_staff",
                "first_name",
                "last_name",
                "full_name",
            ]
        )



from zs_utils.api.amazon.base_api import AmazonAPI


class GetSellerMarketplaceIdsAPI(AmazonAPI):
    """
    Docs:
    https://developer-docs.amazon.com/sp-api/docs/sellers-api-v1-reference#get-sellersv1marketplaceparticipations
    """

    http_method = "GET"
    resource_method = "sellers/v1/marketplaceParticipations"



import base64
import importlib
import re

import requests
from django.conf import settings
from django.core.files import File

from zs_utils.api.services import ApiRequestLogService
from zs_utils.email_notification import models
from zs_utils.exceptions import CustomException
from zs_utils.json_utils import pretty_json


class EmailServiceException(CustomException):
    pass


class Unisender:
    API_URL = "https://go1.unisender.ru/ru/transactional/api/v1/"
    FROM_EMAIL = "noreply@id.kokocgroup.ru"

    @classmethod
    def send_email(
            cls,
            receivers: list,
            sender: str,
            subject: str,
            message: str,
            files: dict = None,
            message_format: str = "plaintext",
            **kwargs,
    ):
        data = {
            "message": {
                "recipients": [{"email": email} for email in receivers],
                "body": {message_format: message},
                "subject": subject,
                "from_email": cls.FROM_EMAIL,
                "from_name": sender,
                "skip_unsubscribe": 1,
                "attachments": [
                    {"name": name, "content": value, "type": "application/octet-stream"}
                    for name, value in files.items()
                ]
                if files
                else None,
            }
        }

        response = requests.post(
            url=cls.API_URL + "email/send.json",
            headers={"X-API-KEY": getattr(settings, "UNISENDER_API_KEY")},
            json=data,
        )
        ApiRequestLogService.save_api_request_log_by_response(response=response, save_if_is_success=False)
        if not response.ok:
            raise EmailServiceException(pretty_json(response.json()))

        return response.json()

    @classmethod
    def send_html_email(cls, **kwargs):
        return cls.send_email(message_format="html", **kwargs)


class EmailService(Unisender):
    """
    Сервис для отправки писем по шаблону через Unisender
    """

    # ------------------------------ Генерация полного HTML шаблона ------------------------------
    @classmethod
    def get_template_file(cls, template_name: str):
        """
        Получение файла с частями шаблона
        """
        template_package = getattr(settings, "EMAIL_TEMPLATE_PACKAGE")
        if not template_package:
            raise EmailServiceException("Не указан базовый пакет шаблонов в настройках")

        full_template_module = f"{template_package}.{template_name}"

        try:
            template_module = importlib.import_module(full_template_module)
        except ModuleNotFoundError:
            raise EmailServiceException(f"Шаблон {template_name} не найден")

        return template_module

    @classmethod
    def get_template_languages(cls, template_name: str) -> list:
        """
        Получение всех языков шаблона
        """
        template_file = cls.get_template_file(template_name=template_name)
        languages = list()
        for part in dir(template_file):
            if "body" in part:
                languages.append(part.split("_")[-1])
        return languages

    @classmethod
    def validate_template_params(cls, html_template: str, template_params: dict):
        """
        Валидация параметров шаблона
        """
        params = re.findall(pattern=r"{(.*?)}", string=html_template)
        required_params = [template_param.replace("{", "") for template_param in params]

        errors = dict()
        required_params = list(set(required_params))
        for required_param in required_params:
            if required_param not in template_params:
                errors[f"template_params.{required_param}"] = "Обязательное поле."
        if errors:
            raise EmailServiceException(message_dict=errors)

    @classmethod
    def get_template_data(cls, template_name: str, language: str, template_params: dict) -> dict:
        """
        Получение данных шаблона
        """
        # Базовая информация: файл шаблона с частями и url со статикой
        template_file = cls.get_template_file(template_name=template_name)
        static_url = getattr(settings, "EMAIL_STATIC_FOLDER_URL", "https://storage.yandexcloud.net/zs-static/email/")
        if not hasattr(template_file, f"body_{language}"):
            raise EmailServiceException(
                message_dict={"language": f"Для данного шаблона не определён язык '{language}'"}
            )

        # Костыль для шаблона zonesmart.order
        if "items" in template_params:
            items = ""
            item_template = getattr(template_file, f"items_{language}", getattr(template_file, "items", None))
            for item in template_params["items"]:
                items += item_template.format(**item)
            template_params["items"] = items

        # Переносим части шаблона в словарь
        title = getattr(template_file, f"title_{language}")
        template_data = {
            "title": title,
            "subject": getattr(template_file, f"subject_{language}", title).format(**template_params),
            "body": getattr(template_file, f"body_{language}"),
            "cheers": getattr(template_file, f"cheers_{language}", ""),
            "footer": getattr(template_file, f"footer_{language}", ""),
            "logo": template_file.base.logo,
            "email_icon": getattr(template_file, "icon", ""),
            "static_url": static_url,
        }

        # Составление полного шаблона из частей
        base_template = template_file.base.base_template
        template_data["html"] = base_template.format(**template_data)
        cls.validate_template_params(html_template=template_data["html"], template_params=template_params)
        template_data["html"] = template_data["html"].format(**template_params)
        return template_data

    @classmethod
    def create_email_notification(
        cls,
        sender: str,
        receivers: list,
        template_name: str,
        language: str = "ru",
        is_urgent: bool = False,
        params: dict = None,
        files: list = None,
    ) -> models.EmailNotification:
        """
        Создание объекта email уведомления
        """
        email_notification = models.EmailNotification.objects.create(
            sender=sender,
            receivers=receivers,
            language=language,
            template_name=template_name,
            template_params=params,
            is_urgent=is_urgent,
        )
        if files:
            for file in files:
                models.EmailNotificationFile.objects.create(
                    email_notification=email_notification, name=file.name, file=File(file)
                )
        return email_notification

    # ------------------------------ Отправка email письма ------------------------------

    @classmethod
    def send_email_notification(cls, email_notification: models.EmailNotification, force: bool = False, **kwargs):
        """
        Отправка email-уведомления по объекту email_notification
        """
        if email_notification.is_sent and not force:
            raise EmailServiceException("Уведомление уже было отправлено.")

        files = dict()
        for file in email_notification.files.all():
            files[file.name] = base64.b64encode(file.file.read()).decode()

        try:
            results = cls.send_template_email(
                sender=email_notification.sender,
                receivers=email_notification.receivers,
                template_name=email_notification.template_name,
                language=email_notification.language,
                params=email_notification.template_params,
                files=files,
                **kwargs,
            )
        except EmailServiceException:
            email_notification.delete()
            raise

        email_notification.is_sent = True
        email_notification.save(update_fields=["is_sent"])
        return results

    @classmethod
    def send_template_email(
        cls,
        template_name: str,
        language: str,
        sender: str,
        receivers: list,
        params: dict,
        files: dict = None,
        **kwargs,
    ):
        """
        Отправка email-уведомления по шаблону
        """
        language = language if language == "ru" else "en"
        template_data = cls.get_template_data(template_name=template_name, language=language, template_params=params)
        return cls.send_html_email(
            sender=sender,
            receivers=receivers,
            subject=template_data["subject"],
            message=template_data["html"],
            files=files,
        )



from zs_utils.api.amazon.base_api import AmazonAPI


class CreateReportAPI(AmazonAPI):
    """
    Docs:
    https://developer-docs.amazon.com/sp-api/docs/reports-api-v2021-06-30-reference#post-reports2021-06-30reports
    """

    http_method = "POST"
    resource_method = "reports/2021-06-30/reports"
    required_params = ["payload"]


class GetReportAPI(AmazonAPI):
    """
    Docs:
    https://developer-docs.amazon.com/sp-api/docs/reports-api-v2021-06-30-reference#get-reports2021-06-30reportsreportid
    """

    http_method = "GET"
    resource_method = "reports/2021-06-30/reports/{reportId}"
    required_params = ["reportId"]


class GetReportDocumentAPI(AmazonAPI):
    """
    Docs:
    https://developer-docs.amazon.com/sp-api/docs/reports-api-v2021-06-30-reference#get-reports2021-06-30documentsreportdocumentid
    """

    http_method = "GET"
    resource_method = "reports/2021-06-30/documents/{reportDocumentId}"
    required_params = ["reportDocumentId"]
